% Author: Yanting Wu

%% basic parameters
global t M max_single_arrival max_idle max_burst_duration;

M=200; % transimission window size
delay_arr=1:M; % delay array: calculate delays for scheduling packets to different slots; make delay array twice of transition window size to make sure all packets are scheduled
K=1000; % simulation rounds
max_single_arrival= 20; % the maximum number of packets arrived at a single slot
max_idle=50; % the maximum number of slots between two arrivals
max_burst_duration=5;%the maximum number of slots last for a single bursty traffic
 
% energy cost function
% exponential case
a=2;  
fc=@(x) exp(a*x)-1;
fc_inv=@(y) log(y+1)/a;
%polynomical case
% fc=@(x) 0.5*(x^2);
% fc_inv=@(y) sqrt(2*y);

% cost function
ft=@(x) sum ( (x).*(delay_arr)+fc((x)));

% Run the simulation multiple times
worst_ratio=0; %worst case competitve ratio
total_ratio=0; %sum of K rounds competitive ratio
for kk=1:K

%% packet arrival sequence generator
t=zeros(M,1);

pattern_selector=rand(1);
% burst arrival
if pattern_selector < 0.2 
    generate_burst_arrival;
% const arrival
elseif pattern_selector>=0.2 && pattern_selector<0.4
    generate_const_arrival;
% random arrival
else
    generate_rand_arrival;
end

%% offline 
cost_off = 0; %total cost for offline scheduling
Noff = zeros(1,M); %offline scheduling results

for ii=M:-1:1
    Npkt = t(ii,1);
    if Npkt ==0
        continue;
    end
    for jj=1:Npkt
        delta_ = delay_arr-ii+fc(Noff+1)-fc(Noff);
        % display(delta_);
        [dfi, ti]=min(delta_(ii:M));
        Noff(1,ti+ii-1)=Noff(1,ti+ii-1)+1;
        cost_off = cost_off+dfi;
        % display(cost_off);
    end 
end 

%% online 
Non=zeros(1,M); %online scheduling results
N_remain=0; %the number of unscheduled packets
cost_on=0; %total cost for online scheduling
N_sent = 0; % the number of packets sent so far
for ii=1:M
    N_remain = N_remain+t(ii);
    Non(ii)=min(N_remain,max(floor(fc_inv(N_remain+1)),1));
    N_remain = N_remain-Non(ii);
    cost_on=cost_on+fc(Non(ii))+N_remain;
end 

ratio=cost_on/cost_off; %competitive ratio
if ratio>worst_ratio
    worst_ratio=ratio;
end;
total_ratio=total_ratio+ratio;
end
avg_ratio=total_ratio/K; %average competitive ratio

display(worst_ratio);
display(avg_ratio);